#include "chromosome.h"
#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <vector>
#include <cmath>
#include <algorithm>

using namespace std;

const int num_generations = 100000;
const int MAX_POP = 200;

bool compare(const Chromosome* a, const Chromosome* b)
{
	return *a < *b;
}

vector<Chromosome *> population;

int random(int max)
{
	return rand() % max;
}

//Hamburger, Cheeseburger, McRib, Grilled Chicken Club Sandwich, Small French Fries, Ketchup Packet, Chicken McNuggets (4 piece), BBQ Sauce, Southwest Salad w/Chicken, Side Salad, Balsalmic Vinagrette Dressing
const string names[] = {"Hamburger", "Cheeseburger", "McRib", "Grilled Chicken Sandwich", "French Fries", "Ketchup Packet", "McNuggets", "BBQ Sauce", "Chicken Salad", "Side Salad", "Balsalmic Vinagrette Dressing"};
int Cals[] = {250,300,500,460,230, 15,190, 50,320, 20, 40};
int Na[] =   { 22, 31, 41, 43,  7,  5, 15, 11, 40,  0, 30};
int Fib[] =  {  6,  7, 10, 15, 12,  0,  2,  0, 25,  6,  0};
int vA[] =   {  0,  6,  2,  8,  0,  2,  0,  2,160, 45,  0};
int vC[] =   {  2,  2, 15,  8,  8,  2,  2,  0, 35, 25,  4};
int Ca[] =   { 10, 20, 15, 30,  2,  0,  0,  0, 15,  2,  0};
int Fe[] =   { 15, 15, 20, 20,  4,  0,  2,  0, 15,  4,  0};

void print_chromosome(Chromosome* c)
{
	int calories = 0;
	int sodium = 0;
	int fiber = 0;
	int vitA = 0;
	int vitC = 0;
	int calcium = 0;
	int iron = 0;


	for (int i=0; i < SIZE; ++i) {
		int count = c->gene(i);
		calcium += Ca[i]*count;
		fiber += Fib[i]*count;
		sodium += Na[i]*count;
		iron += Fe[i]*count;
		vitA += vA[i]*count;
		vitC += vC[i]*count;
		calories += Cals[i]*count;
	}
	for (int i=0; i < SIZE; ++i) {
		cout << "\t" << names[i] << ": " << c->gene(i) << endl;
	}
	cout << "Calories: " << calories << endl << "Sodium: " << sodium << endl << "Dietary Fiber: " << fiber << endl << "Vitamin A: " << vitA << endl;
	cout << "Vitamin C: " << vitC << endl << "Calcium: " << calcium << endl << "Iron: " << iron << endl;
}

double fitness(Chromosome* c){
	int calories = 0;
	int sodium = 0;
	int fiber = 0;
	int vitA = 0;
	int vitC = 0;
	int calcium = 0;
	int iron = 0;


	for (int i=0; i < SIZE; ++i) {
		int count = c->gene(i);
		calcium += Ca[i]*count;
		fiber += Fib[i]*count;
		sodium += Na[i]*count;
		iron += Fe[i]*count;
		vitA += vA[i]*count;
		vitC += vC[i]*count;
		calories += Cals[i]*count;
	}

	int fit = calories;

	if (calcium < 100) {
		fit += 5000;
	}
	if (fiber < 100) {
		fit += 5000;
	}
	if (sodium < 100) {
		fit += 5000;
	}
	if (vitA < 100) {
		fit += 5000;
	}
	if (vitC < 100) {
		fit += 5000;
	}
	if (iron < 100) {
		fit += 5000;
	}
	return fit;
}

int main()
{
	srandom(7);
	for (int i=0; i < MAX_POP; ++i) {
		population.push_back(new Chromosome);
	} 		
	for (int generation = 0; generation < num_generations; ++generation) {
		int size = population.size();
		if (random() % 10000 < 800) {
			int victim = random() % size;
			Chromosome* c = population[victim];
			population.push_back(c->mutate());
		}
		for (int i=0; i < size; ++i) {
			int victim2 = random() % size;

			Chromosome* c1 = population[i];
			Chromosome* c2 = population[victim2];

			population.push_back(c1->crossover(c2));
			population.push_back(c2->crossover(c1));
		}
		int min = 0;
		int max = 0;
		int maxValue = fitness(population[0]);
		int minValue = maxValue;
		double sum = 0.0;
		
		//Compute fitness of all chromosomes
		for (int i=0; i < population.size(); ++i) {
			double fit = fitness(population[i]);	
			population[i]->fitness = fit;

			//Also keep track of minimum and maximum fitnesses
			if (fit < minValue) {
				min = i;
				minValue = fit;
			}
			if (fit > maxValue) {
				max = i;
				maxValue = fit;
			}
			sum += fit;
		} 		

		double avg = sum / population.size();

		//Prune population back to MAX_POP chromosomes
		sort(population.begin(), population.end(), compare);
		if (generation % 10000 == 0) {
			cout << "Generation: " << generation << " Size: " << population.size() << " Worst: " << maxValue << " Average Fitness: " << avg << " Best: " << minValue << endl;
			//		print_chromosome(population[max]);
		}
		while (population.size() > MAX_POP) {
			delete population.back();
			population.pop_back();
		}
	}
	int min = 0;
	for (int i=0; i < population.size(); ++i) {
		double fit = fitness(population[i]);	
		population[i]->fitness = fit;
		if (fit < population[min]->fitness) {
			min = i;
		}
	} 		
	cout << "Best diet: " << endl;
	print_chromosome(population[min]);
}
